// Copyright 2015 - Present, The Gogs Authors. All rights reserved.
// Copyright 2018 - Present, Gitote. All rights reserved.
//
// This source code is licensed under the MIT license found in the
// LICENSE file in the root directory of this source tree.

package repo

import (
	"gitote/gitote/models"
	"gitote/gitote/pkg/context"

	"gitlab.com/gitote/com"
	api "gitlab.com/gitote/go-gitote-client"
)

// ListLabels list all the labels of a repository
func ListLabels(c *context.APIContext) {
	labels, err := models.GetLabelsByRepoID(c.Repo.Repository.ID)
	if err != nil {
		c.Error(500, "GetLabelsByRepoID", err)
		return
	}

	apiLabels := make([]*api.Label, len(labels))
	for i := range labels {
		apiLabels[i] = labels[i].APIFormat()
	}
	c.JSON(200, &apiLabels)
}

// GetLabel get label by repository and label id
func GetLabel(c *context.APIContext) {
	var label *models.Label
	var err error
	idStr := c.Params(":id")
	if id := com.StrTo(idStr).MustInt64(); id > 0 {
		label, err = models.GetLabelOfRepoByID(c.Repo.Repository.ID, id)
	} else {
		label, err = models.GetLabelOfRepoByName(c.Repo.Repository.ID, idStr)
	}
	if err != nil {
		if models.IsErrLabelNotExist(err) {
			c.Status(404)
		} else {
			c.Error(500, "GetLabelByRepoID", err)
		}
		return
	}

	c.JSON(200, label.APIFormat())
}

// CreateLabel create a label for a repository
func CreateLabel(c *context.APIContext, form api.CreateLabelOption) {
	if !c.Repo.IsWriter() {
		c.Status(403)
		return
	}

	label := &models.Label{
		Name:   form.Name,
		Color:  form.Color,
		RepoID: c.Repo.Repository.ID,
	}
	if err := models.NewLabels(label); err != nil {
		c.Error(500, "NewLabel", err)
		return
	}
	c.JSON(201, label.APIFormat())
}

// EditLabel modify a label for a repository
func EditLabel(c *context.APIContext, form api.EditLabelOption) {
	if !c.Repo.IsWriter() {
		c.Status(403)
		return
	}

	label, err := models.GetLabelOfRepoByID(c.Repo.Repository.ID, c.ParamsInt64(":id"))
	if err != nil {
		if models.IsErrLabelNotExist(err) {
			c.Status(404)
		} else {
			c.Error(500, "GetLabelByRepoID", err)
		}
		return
	}

	if form.Name != nil {
		label.Name = *form.Name
	}
	if form.Color != nil {
		label.Color = *form.Color
	}
	if err := models.UpdateLabel(label); err != nil {
		c.Handle(500, "UpdateLabel", err)
		return
	}
	c.JSON(200, label.APIFormat())
}

// DeleteLabel delete a label for a repository
func DeleteLabel(c *context.APIContext) {
	if !c.Repo.IsWriter() {
		c.Status(403)
		return
	}

	if err := models.DeleteLabel(c.Repo.Repository.ID, c.ParamsInt64(":id")); err != nil {
		c.Error(500, "DeleteLabel", err)
		return
	}

	c.Status(204)
}
